#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/mman.h>
#include <sys/resource.h>

#include <linux/i2c.h>
#include <linux/i2c-dev.h>

static int zone = 1;

int verbose = 0;
#define LOG 0

#define ARR_SZ	16
#define ARR_CUT	4

static short temps[5][ARR_SZ];
static short humids[5][ARR_SZ];
static int idx[5] = { -1, -1, -1, -1, -1 };

static int twiddles[6];

static int
get_status (int zone)
{
  char msgbuf[3];
  i2c(zone, 2, 0, msgbuf, 1);
  return msgbuf[0];
}

static int
short_cmp (void *va, void *vb)
{
  short a = *(short *)va;
  short b = *(short *)vb;
  return a - b;
}

static short
mavg(short *l)
{
  static int least_count = ARR_SZ + 2;
  short a[ARR_SZ];
  int sum = 0, i, count=0;
  int min, max;

  memcpy (a, l, sizeof(a));
  qsort (a, ARR_SZ, sizeof(a[0]), short_cmp);

  min = a[ARR_SZ/2] - 10*256;
  max = a[ARR_SZ/2] + 10*256;

  for (i=0; i < ARR_SZ; i++)
    {
      int b = a[i];
      if (b < min || b > max)
	{
	  printf("\033[35m%7d", b);
	  continue;
	}
      printf("\033[0m%7d", b);
      sum += b;
      count ++;
    }
  sum = sum / count;

  printf("\033[0m =%7d     \n", sum);

  if (count < least_count)
    {
      least_count = count;
      printf("\033[20;60H%5d", least_count);
    }

  return sum;
}

static void
get_climate (int zone)
{
  int i;
  short it, ih;
  float t, h, f;
  unsigned char th[4];

  i2c(zone, 3, 0, th, 4);

  it = (th[0] << 8) | th[1];
  ih = (th[2] << 8) | th[3];

  if (idx[zone] == -1)
    {
      for (i=0; i<ARR_SZ; i++)
	{
	  temps[zone][i] = it;
	  humids[zone][i] = ih;
	}
      idx[zone] = 0;
    }
  else
    {
      i = idx[zone];
      temps[zone][i] = it;
      humids[zone][i] = ih;
      idx[zone] = (i+1) % ARR_SZ;
      printf("\033[%d;1H", zone*4+1);
      it = mavg(temps[zone]);
      printf("\033[%d;1H", zone*4+2);
      ih = mavg(humids[zone]);
    }

  t = it/256.0;
  h = ih/256.0;

  f = t * 9.0/5.0 + 32.0;

#if LOG
    printf("\033[%dm[%d] climate: %04x %04x %.2fC %.2fF %.2f%%\033[0m\n",
	   zone+30, zone, it, ih, t, f, h);
#else
  if (!verbose)
    printf("\033[%d;1H\033[%dmclimate: %.2fC %.2fF %.2f%%  %c",
	   zone*4, zone+30, t, f, h,
	   "\\|/-"[(twiddles[zone]++)&3]);
#endif
  fflush (stdout);
}

static void
check_ac_clock ()
{
  unsigned char th[4];
  unsigned long up;
  int d, h, m, s, ms;

  i2c(5, 7, 0, th, 4);

  up = (th[3] << 24) | (th[2] << 16) | (th[1] << 8) | th[0];

  d =   up / (24*60*60*60);
  h =  (up / (60*60*60)) % 24;
  m =  (up / (60*60))    % 60;
  s =  (up /  60)        % 60;
  ms = (up % 60) * 100 / 60;

  printf("\033[36m\033[20;1H %02x %02x %02x %02x   %2d %2d:%02d:%02d.%02d  %lu",
	 th[3], th[2], th[1], th[0],
	 d, h, m, s, ms, up);
  fflush (stdout);
}

int
main(int argc, char **argv)
{
  int s, os[6];
  int i, ts, te, z;

  for (i=0; i<4; i++)
    os[i] = 0;

  verbose = 0;

  //setbuf(stdout, 0);
  i2c_open ();

#if !LOG
  if (!verbose)
    printf("\033[1;1H\033[2J");
#endif

  i = 0;
  ts = msclock();
  while (1)
    {
      mspause(10);

      te = msclock ();
      if (te - ts > 100000)
	{
	  ts = te;
	  check_ac_clock ();
	}

      for (z=1; z<=4; z++)
	{
	  s = get_status (z);

	  if (s & 0x80)
	    get_climate(z);

	  if ((s & 0x0f) == (os[z] & 0x0f))
	    continue;

#if LOG
	  //	  printf("[%d] status %02x", z, s);
#else
	  if (!verbose)
	    printf("\033[%d;60H%02x", z*4, s);
#endif

	  {
	    int b;
	    int c = s ^ os[z];
	    if (!verbose)
	      for (b=0; b<4; b++)
		if (c & (1<<b))
#if LOG
		  printf("[button %d.%d %s]\n", z, b, s & (1<<b) ? "pressed" : "released");
#else
		  printf("\033[%d;40Hbutton %d %s", z*4, b, s & (1<<b) ? "pressed" : "released");
#endif

	  }
#if LOG
	  //	  printf("\n");
#else
	  if (!verbose)
	    printf("\033[25;7H");
#endif
	  fflush (stdout);
	  os[z] = s;
	}
    }

  i2c_close ();
  return 0;
}
