#include <stdio.h>
#include <getopt.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/mman.h>
#include <sys/resource.h>

static char *gpio_base;
#define GPIO_BASE	0x40e00000
#define MAP_SIZE	4096
#define MAP_MASK	( MAP_SIZE - 1 )
#define gpio_unit	((volatile int *)(gpio_base))
#define GPIO_GPLR	((volatile int *)(gpio_base+0x00))
#define GPIO_GPDR	((volatile int *)(gpio_base+0x0c))
#define GPIO_GPSR	((volatile int *)(gpio_base+0x18))
#define GPIO_GPCR	((volatile int *)(gpio_base+0x24))
#define GPIO_AFR	((volatile int *)(gpio_base+0x54))


void
gpio_setup ()
{
  int fd = open ("/dev/mem", O_RDWR | O_SYNC);
  if (fd < 0) {
    perror ("open(\"/dev/mem\")");
    exit (1);
  }
  gpio_base = mmap ((void *)GPIO_BASE,
		    MAP_SIZE,
		    PROT_READ | PROT_WRITE,
		    MAP_SHARED,
		    fd,
		    GPIO_BASE & ~MAP_MASK
		    );
  if (gpio_base == (char *) -1 ) {
    perror ("mmap()");
    exit (1);
  }
}

void
gpio_set_mode (int gpio, int dir, int af)
{
  if (dir)
    GPIO_GPDR[gpio/32] |=  (1 << (gpio % 32));
  else
    GPIO_GPDR[gpio/32] &= ~(1 << (gpio % 32));
  GPIO_AFR[gpio/16] &= ~(3<<((gpio%16)*2));
  GPIO_AFR[gpio/16] |=  (af<<((gpio%16)*2));
}

int
gpio_read (int gpio)
{
  return GPIO_GPLR[gpio/32] & (1 << (gpio % 32)) ? 1 : 0;
}

void
gpio_write (int gpio, int bit)
{
  if (bit)
    GPIO_GPSR[gpio/32] |= (1 << (gpio % 32));
  else
    GPIO_GPCR[gpio/32] |= (1 << (gpio % 32));
}

void
mspause(int ms)
{
  struct timeval s, e;
  gettimeofday(&s, 0);
  /*printf("%3d %d.%06d : ", ms, s.tv_sec, s.tv_usec);*/
  s.tv_usec += ms * 1000;
  if (s.tv_usec > 999999)
    {
      s.tv_sec += s.tv_usec / 1000000;
      s.tv_usec %= 1000000;
    }
  while (1)
    {
      gettimeofday(&e, 0);
      if (e.tv_sec > s.tv_sec
	  || (e.tv_sec == s.tv_sec
	      && e.tv_usec >= s.tv_usec))
	{
	  /*printf("%d.%06d\n", e.tv_sec, e.tv_usec);*/
	  return;
	}
    }
}

void
uspause(int us)
{
  struct timeval s, e;
  gettimeofday(&s, 0);
  /*printf("%3d %d.%06d : ", ms, s.tv_sec, s.tv_usec);*/
  s.tv_usec += us;
  if (s.tv_usec > 999999)
    {
      s.tv_sec += s.tv_usec / 1000000;
      s.tv_usec %= 1000000;
    }
  while (1)
    {
      gettimeofday(&e, 0);
      if (e.tv_sec > s.tv_sec
	  || (e.tv_sec == s.tv_sec
	      && e.tv_usec >= s.tv_usec))
	{
	  /*printf("%d.%06d\n", e.tv_sec, e.tv_usec);*/
	  return;
	}
    }
}

int
msclock()
{
  static struct timeval tv;
  static long long basis;
  static int first=1;
  int rv;

  if (first)
    {
      gettimeofday (&tv, 0);
      basis = tv.tv_sec * 1000000LL + tv.tv_usec;
      first = 0;
      return 0;
    }
  gettimeofday (&tv, 0);
  rv = (int)(tv.tv_sec * 1000000LL + tv.tv_usec - basis);
  /*printf("[%d]", rv);*/
  return rv;
}
