#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/mman.h>
#include <sys/resource.h>

#include <linux/i2c.h>
#include <linux/i2c-dev.h>

#include "gpio.h"

extern int verbose;

#define Z1_RESET	64
#define ZP_MODE		61

static int zone_addr[] = { 0, 10, 20, 30, 40, 50 };

main(int argc, char **argv)
{
  int addr, zone = 1;
  struct i2c_msg msg[2];
  struct i2c_rdwr_ioctl_data mlist;
  int i2c, i, wi=0, ri=0;
  char msgbuf[40];
  char rdbuf[40];

  gpio_setup ();

  setbuf(stdout, 0);
  i2c = open("/dev/i2c-0", O_RDWR);
  if (i2c < 0)
    {
      perror ("/dev/i2c-0");
      exit (1);
    }

  ioctl (i2c, I2C_TIMEOUT, 3);
  ioctl (i2c, I2C_RETRIES, 0);

  addr = zone_addr[zone];

  for (i=1; i<argc; i++)
    {
      if (strcmp (argv[i], "-z") == 0)
	{
	  zone = atoi(argv[i+1]);

	  if (zone < 1 || zone > 5)
	    {
	      fprintf(stderr, "invalid zone %d, must be 1..5\n", zone);
	      exit(1);
	    }
	  addr = zone_addr[zone];


	  i ++;
	}
      else if (strcmp (argv[i], "-a") == 0)
	{
	  addr = atoi(argv[i+1]);

	  i ++;
	}
      else if (strcmp (argv[i], "-v") == 0)
	{
	  verbose ++;
	}
      else if (strcmp (argv[i], "-w") == 0)
	{
	  wi = 0;
	}
      else if (strcmp (argv[i], "-r") == 0
	       || strcmp (argv[i], "r") == 0)
	{
	  ri = atoi(argv[i+1]);;
	  i++;
	}
      else
	{
	  msgbuf[wi++] = strtol(argv[i], 0, 0);
	}
    }
  msg[0].addr = addr;
  msg[0].flags = 0;
  msg[0].len = wi;
  msg[0].buf = msgbuf;

  msg[1].addr = addr;
  msg[1].flags = I2C_M_RD;
  msg[1].len = ri;
  msg[1].buf = rdbuf;

  mlist.msgs = msg;
  mlist.nmsgs = ri ? 2 : 1;

  memset (rdbuf, 0, sizeof(rdbuf));
  if (ioctl (i2c, I2C_RDWR, &mlist) < 0)
    {
      perror("ioctl");
      exit(1);
    }

  if (ri)
    {
      printf("read:");
      for (i=0; i<ri; i++)
	printf(" %02x", rdbuf[i]);
      if (ri == 1)
	{
	  printf(" ");
	  for (i=0; i<8; i++)
	    {
	      printf("%d", rdbuf[0] & (0x80 >> i) ? 1 : 0);
	      if (i == 3)
		printf(".");
	    }
	}
      printf("\n");
    }

  close (i2c);
  return 0;
}
