.include "r8c-ports.S"


;;; Port 1 has the I/O pins.  The ones we're using are:
;;; P1.7 - output - drive
;;; P1.5 - input - sense

;;; The timers are driven off f1, 20MHz.  We normally prescale that by
;;; 20, resulting in a 1uS-per-count interval.  For the reset pulse,
;;; we prescale it by 200 to get a 10uS-per-count interval.
;;; We're using timer Z in one-shot mode.

#define IPTRACE	mov.w	#__LINE__,iptrace

	.data
ow_done:
	.byte	1
ow_rv:
	.byte	0xff
	.byte	0

	.text

	.global	_ow_init
_ow_init:
	mov.b	#0xE0,tzmr	; one-shot mode
	mov.b	#0x00,pum	; ignore INT0, high pulse
	mov.b	#20,prez	; 1uS per count prescale
	mov.b	#0,tzoc		; one-shot stops
	mov.b	#0x00,tcss	; X f1, Z f1
	mov.b	#7,tzic		; enable interrupts
	and.b	#0x7f,_p1_bits	; L
	mov.b	_p1_bits,p1
	bclr	7,pd1		; off, pulled high
	rts

;;; On general principles, we do NOT wait for the current task to be
;;; done before we return to the caller.  We let it keep going in the
;;; background, but wait for it to finish before STARTING something
;;; else.  This way, we can get the most throughput out.  Also, this
;;; means that the "read" call can return to the caller as soon as it
;;; has the bit read.

ow_wait:
1:	cmp.b	#0,ow_done	; is ow_done set?
	jeq	1b		; no, loop
	mov.b	#0,ow_done	; clear it for the next op
	rts

;;; Wait for one of the ISRs to set the ow_rv byte to something
;;; other than 0xff, then return that value.

ow_rvwait:
1:	mov.b	ow_rv,r0l
	cmp.b	#0xff,r0l
	jeq	1b
	mov.b	#0,r0h
	rts

;;; All the timer routines use this as the last step, except reset, which has
;;; to reprogram the prescaler.
ow_set_done:
	mov.b	#1,ow_done
	reit

;;; -------------------------------------------------------------------------
	.global	_ow_reset
_ow_reset:
	jsr.w	ow_wait
	mov.b	#0xff,ow_rv
	mov.w	#ow_reset_2, ivect_timerz
	mov.b	#200,prez
	mov.b	#47,tzpr
	bset	7,pd1		; low
	bset	0,tzoc		; start counter
;	and.b	#0xfe,_p1_bits	; red
;	mov.b	_p1_bits,p1
	jmp.w	ow_rvwait
	
ow_reset_2:
	bclr	7,pd1		; high
;	or.b	#0x01,_p1_bits	; red
;	mov.b	_p1_bits,p1
	mov.b	#5,tzpr
	mov.w	#ow_reset_3, ivect_timerz
	bset	0,tzoc		; start counter
	reit
	
ow_reset_3:
	btst	5,p1
	stzx	#0,#1,ow_rv
	mov.b	#52,tzpr
	mov.w	#ow_reset_done, ivect_timerz
	bset	0,tzoc		; start counter
	reit

ow_reset_done:
	mov.b	#20,prez
	mov.b	#1,ow_done
	reit

;;; -------------------------------------------------------------------------
	.global	ow_write_0
ow_write_0:
	jsr.w	ow_wait
	mov.b	#0xff,ow_rv
	mov.w	#ow_write_0b, ivect_timerz
	mov.b	#15,tzpr
	bset	7,pd1		; low
	bset	0,tzoc		; start counter
	jmp.b	ow_rvwait

ow_write_0b:
	btst	5,p1
	stzx	#0,#1,ow_rv
	mov.w	#ow_write_0c, ivect_timerz
	mov.b	#59,tzpr
	bset	_tzos,tzoc	; start counter
	reit

ow_write_0c:
	bclr	7,pd1		; off, high
	mov.w	#ow_set_done, ivect_timerz
	mov.b	#43+20,tzpr
	bset	_tzos,tzoc	; start counter
	reit

;;; -------------------------------------------------------------------------
	.global	ow_write_1
ow_write_1:
	jsr.w	ow_wait
	mov.b	#0xff,ow_rv
	mov.w	#ow_write_1b, ivect_timerz
	mov.b	#6,tzpr
	bset	7,pd1		; low
	bset	0,tzoc		; start counter
	jmp.b	ow_rvwait

ow_write_1b:
	bclr	7,pd1		; off, high
	or.b	#0x80,_p1_bits	; H
	mov.b	_p1_bits,p1
	bset	7,pd1		; on, high
	pushm	r0,r1
	popm	r0,r1
	bclr	7,pd1		; off, high
	and.b	#0x7f,_p1_bits	; L
	mov.b	_p1_bits,p1
	mov.w	#ow_write_1c, ivect_timerz
	mov.b	#12,tzpr
	bset	_tzos,tzoc	; start counter
	reit

ow_write_1c:
	btst	5,p1
	stzx	#0,#1,ow_rv
	mov.w	#ow_set_done, ivect_timerz
	mov.b	#79+20,tzpr
	bset	_tzos,tzoc	; start counter
	reit

;;; -------------------------------------------------------------------------

	.text
	.global	_ow_read
_ow_read:
	jsr.w	ow_wait
;	xor.b	#0x17,_p1_bits
;	mov.b	_p1_bits,p1
	mov.b	#0xff,ow_rv
	mov.w	#ow_read_2, ivect_timerz
	mov.b	#2,tzpr
	bset	0,tzoc		; start counter
	bset	7,pd1		; low
	jsr.w	ow_rvwait
	push.w	r0
	add.b	#'0',r0l
;	jsr.w	console_putc
	pop.w	r0
	rts

ow_read_2:
	bclr	7,pd1		; off, high
	or.b	#0x80,_p1_bits	; H
	mov.b	_p1_bits,p1
	bset	7,pd1		; on, high
	pushm	r0,r1,r2,r3	; delays
	popm	r0,r1,r2,r3
	bclr	7,pd1		; off, high
	and.b	#0x7f,_p1_bits	; L
	mov.b	_p1_bits,p1
	mov.w	#ow_read_3, ivect_timerz
	mov.b	#4,tzpr
	bset	0,tzoc		; start counter
	reit

ow_read_3:
	btst	5,p1
	stzx	#0,#1,ow_rv
;	xor.b	#0x17,_p1_bits
;	mov.b	_p1_bits,p1
	mov.w	#ow_set_done, ivect_timerz
	mov.b	#87+20,tzpr
	bset	_tzos,tzoc	; start counter
	reit

;;; -------------------------------------------------------------------------

;;; byte to write is in r1l
	.global	_ow_write_byte
_ow_write_byte:
	mov.w	#0,r3
	mov.b	#8,r1h
3:	btst	0,r1
	jz	1f
	IPTRACE
	jsr.w	ow_write_1
	jmp.b	2f
1:
	IPTRACE
	jsr.w	ow_write_0
2:
	IPTRACE
	rot.b	#-1,r1l
	or.w	r0,r3		; r0 is #0 or #1
	rot.w	#-1,r3
	add.b	#-1,r1h
	jnz	3b
	rot.w	#8,r3
	mov.w	r3,r0
	IPTRACE
	rts

;;; -------------------------------------------------------------------------

;;; byte read is returned in r0l
	.global	_ow_read_byte
_ow_read_byte:
	mov.b	#8,r1h
	mov.b	#0,r1l
3:	jsr.w	_ow_read
	or.b	r0l,r1l
	rot.b	#-1,r1l
	add.b	#-1,r1h
	jnz	3b
	mov.b	r1l,r0l
	mov.b	#0,r0h
	rts

;;; -------------------------------------------------------------------------

;;; pointer in r1, length in r2
	.global	_ow_write_block
_ow_write_block:
	mov.w	r1,a0
1:	
	cmp.w	#0,r2
	jeq	1f
	mov.b	[a0],r1l
	add.w	#-1,r2
	add.w	#1,a0
	jsr.w	_ow_write_byte
	jmp.b	1b
1:	rts

;;; pointer in r1, length in r2
	.global	_ow_read_block
_ow_read_block:
	mov.w	r1,a0
1:	
	cmp.w	#0,r2
	jeq	1f
	jsr.w	_ow_read_byte
	mov.b	r1l,[a0]
	add.w	#-1,r2
	add.w	#1,a0
	jmp.b	1b
1:	rts

;;; -------------------------------------------------------------------------

;;; search stuff.

	.bss

	;; bit 0 is the bit we wrote last time through.
	;; bit 1 is set if there was an option at this bit.
	;; bit 2 is set if this is an unknown
	.global	state
state:	.space	64
state_end:	

	;; this is the address we found with each search.
	.global	_ow_search_addr
_ow_search_addr:
	.space	8

	;; if nonzero, a crc error was detected during the scan.
	.global	_ow_search_error
_ow_search_error:
	.space	1

	.text
;;; pass the number of bytes of _ow_search_addr to be a prefix for
;;; the search in r1l; zero means find everything.
	.global	_ow_search_init
_ow_search_init:
	mov.b	#0,_ow_search_error
	mov.w	#state,a1
	mov.w	#64,r3
	mov.b	#4,r0l
	sstr.b

	mov.w	#state,a1
	mov.w	#_ow_search_addr,a0
ow_search_init_loop:	
	cmp.b	#0,r1l
	jne	1f
	rts

1:	mov.b	[a0],r0l
	mov.b	#8,r0h
	
2:	mov.b	r0l,r1h		; copy one bit to state
	rot.b	#-1,r0l
	and.b	#1,r1h
	mov.b	r1h,[a1]
	
	add.w	#1,a1		; bump state pointer
	add.b	#-1,r0h
	jnz	2b

	add.w	#1,a0		; bump address pointer
	add.b	#-1,r1l
	jmp.b	ow_search_init_loop

;;; Returns 0 in r0l at end of search.
;;; Returns 1 in r0l and address in _ow_search_addr otherwise.
	.global	_ow_search_next
_ow_search_next:
	IPTRACE
	mov.w	#state,a0
	mov.w	#0,a1
ow_search_next1:
	IPTRACE
	mov.b	[a0],r0l
	cmp.b	#4,r0l
	jeq	ow_search_begin	; we init to 4
	cmp.b	#2,r0l
	jnz	1f
	mov.w	a0,a1		; remember the last 2 we saw
1:	add.w	#1,a0
	cmp.w	#state+64,a0
	jne	ow_search_next1
	;; At this point, we have a1 or we don't
	cmp.w	#0,a1
	jne	3f
	mov.b	#0,r0l		; we don't, search finished
;	jsr.w	console_prints
;	.ascii	"search done\r\n\0"
	IPTRACE
	rts
3:
	;; At this point, we need to "fix" the search.
	IPTRACE
	mov.b	#1,[a1]
fix_top:
	add.w	#1,a1
	cmp.w	#(state+64),a1
	jeq	ow_search_begin
	mov.b	#4,[a1]
	jmp.b	fix_top

ow_search_begin:
;	mov.w	#state,a0
;	mov.w	#64,r2
;	jsr.w	console_nibble_block
;	jsr.w	console_crlf

	IPTRACE
	jsr.w	_ow_reset
	cmp.w	#0,r0
	jeq	1f
	mov.w	#1,_ow_search_error
	mov.w	#0,r0
	rts
1:	
	mov.b	#0xf0,r1l	; search rom
	IPTRACE
	jsr.w	_ow_write_byte
	IPTRACE

	;; ok, for each byte in state we have to do two reads and a
	;; write.  The result of the two reads helps us determine
	;; which write to do.  A0 is the state pointer.

	mov.w	#state,a0
ow_search_next_bit:
	.data
_a1_s1:	.short	0
_a1_s2:	.short	0
	.text
	IPTRACE
	jsr.w	_ow_read	; read all the '0' bits
	mov.b	ow_rv,_a1_s1
	mov.b	r0l,_a1_s1+1
	rot.b	#1,r0l
	mov.b	r0l,a1
	jsr.w	_ow_read	; read all the '1' bits
	mov.b	ow_rv,_a1_s2
	mov.b	r0l,_a1_s2+1
	or.b	r0l,a1

;	mov.b	a1,r0l
;	add.b	#'a',r0l
;	jsr.w	console_putc

	tst.w	#0xfffc,a1
	jz	9f
	jsr.w	console_prints
	.ascii	"\r\na1 not 0..3 : \r\n\0"
	jsr.w	_dumpregs
	mov.w	#_a1_s1,a0
	mov.w	#4,r2
	jsr.w	console_hex_block
	jsr.w	console_crlf
9:	
	
	;; a1 now has (binary):
	;; 00 - decision node
	;; 01 - all zero
	;; 10 - all one
	;; 11 - no devices
	rot.b	#1,a1
	rot.b	#1,a1
	IPTRACE
;	jsr.w	_dumpregs
;	mov.b	#'?',r0l
;	jsr.w	console_putc
1:	jmpi.a	2f[a1]
	.byte	0
2:	.long	ow_snb_decide
	.long	ow_snb_zero
	.long	ow_snb_one
	.long	ow_snb_nothing

ow_snb_decide:
	IPTRACE
;	mov.b	#'^',r0l
;	jsr.w	console_putc
	cmp.b	#4,[a0]		; if it's 4, it's a fresh decision point
	jeq	ow_snb_d4
	cmp.b	#0,[a0]
	jeq	ow_snb_zero
	cmp.b	#1,[a0]
	jz	ow_snb_one
	jsr.w	ow_write_0	; else assume it's a 2, so we redo the '0'
	jmp.b	ow_snb_common

ow_snb_d4:
	IPTRACE
	mov.b	#2,[a0]		; write a zero this time, but tag this zero
	jsr.w	ow_write_0	; so we write a 1 next time.
;	mov.b	#'D',r0l
;	jsr.w	console_putc
	jmp.b	ow_snb_common

ow_snb_zero:
	IPTRACE
;	mov.b	#'<',r0l
;	jsr.w	console_putc
	cmp.b	#1,[a0]
	jz	ow_snb_nothing
	mov.w	a0,r1
	mov.b	#0,[a0]
	jsr.w	ow_write_0
;	mov.b	#'0',r0l
;	jsr.w	console_putc
	jmp.b	ow_snb_common

ow_snb_one:
	IPTRACE
;	mov.b	#'>',r0l
;	jsr.w	console_putc
	cmp.b	#0,[a0]
	jz	ow_snb_nothing
	mov.b	#1,[a0]
	jsr.w	ow_write_1
;	mov.b	#'1',r0l
;	jsr.w	console_putc
	jmp.b	ow_snb_common

ow_snb_nothing:
	IPTRACE
;	mov.b	#'!',r0l
;	jsr.w	console_putc
	;; This only happens if there's *nothing* on the list,
	;; or if something interrupted us.  Either way, we're done.
;	jsr.w	console_prints
;	.ascii	"nothing found\r\n\0"
	mov.b	#0,r0l
	rts

ow_snb_common:
	IPTRACE
	add.w	#1,a0
	cmp.w	#state+64,a0
	jlt	ow_search_next_bit

	mov.w	#state,a1
	mov.w	#_ow_search_addr,a0
	mov.b	#8,r1h		; byte count
	
ow_search_found_loop:	
	IPTRACE
	mov.b	#0,r0l		; temp for addr byte
	mov.b	#8,r0h		; bit count
	
2:	mov.b	[a1],r1l
	rorc.b	r1l		; move bit into carry
	rorc.b	r0l		; and into addr byte

	add.w	#1,a1		; bump state pointer
	add.b	#-1,r0h
	jnz	2b

	mov.b	r0l,[a0]
	add.w	#1,a0		; bump address pointer
	add.b	#-1,r1h
	jnz	ow_search_found_loop
	
;	mov.w	#state,a0
;	mov.w	#64,r2
;	jsr.w	console_nibble_block
;	jsr.w	console_crlf

;	jsr.w	console_prints
;	.ascii	"returning addr: \0"
;	mov.w	#_ow_search_addr,a0
;	mov.w	#8,r2
;	jsr.w	console_hex_block

	IPTRACE
	mov.w	#_ow_search_addr, r1
	mov.w	#8,r2
	jsr.w	_crc
	cmp.b	#0,r0l
	jnz	1f
;	jsr.w	console_prints
;	.ascii " - CRC OK\r\n\0"
	mov.b	#1,r0l
	IPTRACE
	rts
	
1:	
;	jsr.w	console_prints
;	.ascii " - CRC BAD \0"
	mov.b	a1,r0l
	jsr.w	console_print_r0l
	jsr.w	console_crlf
	mov.w	#1,_ow_search_error
	mov.b	#0,r0l
	IPTRACE
	rts
