#include <stdarg.h>

#include "r8c-ports.h"
#include "fix.h"

void
cput_c(int ch)
{
  if (ch == 0xff)
    ch = '$';
  if (ch == '\n')
    cput_c ('\r');
  while (!u1c1.ti)
    ;
  u1tb = ch;
}

cput_crlf ()
{
  cput_c ('\n');
}

void
cput_s (char *s)
{
  while (*s)
    {
      cput_c(*s);
      s++;
    }
}

static char hex[] = "0123456789abcdef";

void
cput_nibble (int n)
{
  cput_c (hex[n&0x0f]);
}

void
cput_hex_byte (int n)
{
  cput_nibble (n >> 4);
  cput_nibble (n);
}

void
cput_hex_word (int n)
{
  cput_hex_byte (n >> 8);
  cput_hex_byte (n);
}

void
cput_hex_block (byte *block, int n)
{
  int i = 0;
  while (n)
    {
      cput_hex_byte (*block++);
      if (--n == 0)
	break;
      i++;
      if (i & 7 == 0)
	cput_c (' ');
      else
	cput_c (':');
    }
}

void
cput_nibble_block (byte *block, int n)
{
  int i = 0;
  while (n)
    {
      cput_nibble (*block);
      if (--n == 0)
	break;
      i++;
      if (i & 7 == 0)
	cput_c (' ');
    }
}

void
cput_number (int n)
{
  char buf[20];
  int i = 0;
  if (n < 0)
    {
      cput_c ('-');
      n = -n;
    }
  while (n > 9)
    {
      buf[i++] = (n%10) + '0';
      n /= 10;
    }
  buf[i++] = (n%10) + '0';
  while (i > 0)
    cput_c (buf[--i]);
}

void
printf (const char *fmt, ...)
{
  va_list v;
  int i;
  char *s;

  va_start (v, fmt);

  while (*fmt)
    {
      if (*fmt != '%')
	cput_c (*fmt);
      else
	switch (*++fmt)
	  {
	  case '%':
	    cput_c ('%');
	    break;
	  case 'c':
	    i = va_arg (v, int);
	    cput_c(i);
	    break;
	  case 'd':
	    i = va_arg (v, int);
	    cput_number(i);
	    break;
	  case 'b':
	    i = va_arg (v, int);
	    cput_hex_byte (i);
	    break;
	  case 'w':
	    i = va_arg (v, int);
	    cput_hex_word (i);
	    break;
	  case 'x':
	    s = va_arg (v, char *);
	    i = va_arg (v, int);
	    cput_hex_block (s, i);
	    break;
	  case 'n':
	    s = va_arg (v, char *);
	    i = va_arg (v, int);
	    cput_nibble_block (s, i);
	    break;
	  case 's':
	    s = va_arg (v, char *);
	    cput_s (s);
	    break;
	  case 'f':
	    {
	      FixType f;
	      f = va_arg (v, FixType);
	      cput_hex_byte (f.i >> 8);
	      cput_hex_byte (f.i & 0xff);
	      cput_c('.');
	      cput_hex_byte (f.f >> 8);
	      cput_hex_byte (f.f & 0xff);
	    }
	    break;
	  case 'F':
	    {
	      FixType f, r;
	      int i;
	      f = va_arg (v, FixType);
	      if (f.i & 0x8000)
		{
		  cput_c('-');
		  f = fix_neg (f);
		}
	      else
		cput_c(' ');
	      r = F(10,0);
	      cput_number (f.i);
	      cput_c ('.');
	      for (i=0; i<3; i++)
		{
		  f.i = 0;
		  f = fix_mul (f, r);
		  cput_c (f.i + '0');
		}
	    }
	    break;
	  }
      fmt ++;
    }
}

volatile pause()
{
  int i;
  for (i=0; i<20; i++)
    asm("");
}

volatile pause2()
{
  int i;
  for (i=0; i<2; i++)
    asm("");
}
