.include "r8c-ports.S"

	.text
	.global	_main
_main:
	mov.b	#0xff, pd1	; output
	mov.b	#0x00, pd3	; input
	mov.b	#0x04, pd4	; input

	mov.b	#0x5a,sar	; slave address 0x5a, i2c format
	mov.b	#0x80,pmr	; port mode - i2c ports
	mov.b	#0x80,iccr1	; i2c enabled

	mov.b	#0x8d,iccr1	; i2c enabled, 100 kHz
	mov.b	#0x00,icmr	; 8 bits, msb first, no wait
	mov.b	#0xe2,icier	; all interrupts

	mov.b	#5,iic2aic	; enable interrupts

foo:
	mov.b	icsr,p1
	jmp.b	foo

	.bss

i2x_tx_base:	.space	2
i2x_tx_end:	.space	2
i2x_tx_ptr:	.space	2
i2x_rx_ptr:	.space	2	; set to NULL when we're waiting for regno
i2c_regno:	.space	1

;;; ICSR:	  TDRE TEND RDRF NACKF   STOP AL AAS ADZ

;;; read mode, we get 22 (RDRF+AAS) for the address byte, then 20
;;; (RDRF) for each other byte, then 08 (STOP).

	.text

i2c_irq_hash:
        .byte   28,0,4,0,8,0,4,0,12,12,12,12,12,12,12,12,24,24,24,24,24,24
        .byte   24,24,12,12,12,12,12,12,12,12,16,16,0,0,16,16,0,0,12,12,0,0
        .byte   12,12,0,0,16,16,0,0,16,16,0,0,12,12,0,0,12,12,0,0,8,8,8,8,8
        .byte   8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,0
        .byte   0,8,8,0,0,8,8,0,0,8,8,0,0,8,8,0,0,8,8,0,0,8,8,0,0,8,8,0,0
        .byte   20,20,4,4,20,20,4,4,12,12,4,4,12,12,4,4,20,20,4,4,20,20,4,4
        .byte   12,12,4,4,12,12,4,4,16,16,0,0,16,16,0,0,12,12,0,0,12,12,0,0
        .byte   16,16,0,0,16,16,0,0,12,12,0,0,12,12,0,0,8,8,4,4,8,8,4,4,8,8
        .byte   4,4,8,8,4,4,8,8,4,4,8,8,4,4,8,8,4,4,8,8,4,4,8,8,0,0,8,8,0,0
        .byte   8,8,0,0,8,8,0,0,8,8,0,0,8,8,0,0,8,8,0,0,8,8,0,0

	.data
i2c_irq_handlers:
	.word	i2c_irq_read_start
	.word	i2c_irq_write_start
	.word	i2c_irq_reit ; read_done
	.word	i2c_irq_write_done
i2c_irq_rx_handler:
	.word	i2c_irq_read_regno
i2c_irq_tx_handler:
	.word	i2c_irq_write_done
	.word	i2c_irq_nack
	.word	i2c_irq_reit

	.text
	.global	irq_i2c
irq_i2c:
	pushm	r1,a0
	mov.b	#0x42,p1
	mov.b	icsr,r1l
	mov.b	r1l,a0
	mov.b	a0,p1
	mov.b	i2c_irq_hash[a0],a0
	jmpi.a	i2c_irq_handlers[a0]

i2c_irq_nack:	
i2c_irq_reit:
	mov.b	#0xff,p1
	mov.b	icsr,p1  
	mov.b	#0,icsr
	popm	r1,a0
	reit

i2c_irq_read_start:
	mov.b	#0x52,p1
	mov.w	#i2c_irq_read_regno,i2c_irq_rx_handler
	jmp.w	i2c_irq_reit
	
i2c_irq_write_start:
	mov.b	#0x51,p1
	mov.b	#0xf0,r1l
1:	add.b	#-1,r1l
	jnz	1b
	mov.b	#0x59,p1
	jmpi.a	i2c_irq_tx_handler

i2c_irq_read_regno:
;	mov.b	#0x11,p1
	mov.b	icdrr,a0	; read data byte
	mov.b	a0,i2c_regno
	shl.w	#2,a0
	jmpi.a	i2c_regno_setup[a0]

i2c_irq_write_done:
	mov.b	#0x61,p1
	mov.b	#0x8d,iccr1	; i2c enabled, 100 kHz
	mov.b	icdrr,r1l	; dummy read
	jmp.w	i2c_irq_reit

i2c_regno_setup:
	.word	i2c_setup_port_1

i2c_setup_port_1:
	mov.b	#0x12,p1
	mov.w	#i2c_irq_read_p1,i2c_irq_rx_handler
	mov.w	#i2c_irq_write_p1,i2c_irq_tx_handler
	mov.b	#0xe2,icier	; all interrupts
	jmp.w	i2c_irq_reit

i2c_irq_read_p1:
	mov.b	icdrr,r1l	; read data byte
	mov.b	r1l,p1		; do something useful with it
	jmp.w	i2c_irq_reit

	.data
flag:	.byte	0xbf
	.text
i2c_irq_write_p1:
;	mov.b	p1,r1l	; read data byte
;	add.b	#1,r1l
	add.b	#1,flag
	mov.b	flag,r1l
	mov.b	r1l,p1		; do something useful with it
	mov.b	r1l,icdrt
	mov.b	#0x62,icier	; shut off tx empty interrupt
	jmp.w	i2c_irq_reit

	.data
